/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla.
 *
 * The Initial Developer of the Original Code is IBM Corporation.
 * Portions created by IBM Corporation are Copyright (C) 2004
 * IBM Corporation. All Rights Reserved.
 *
 * Contributor(s):
 *   Darin Fisher <darin@meer.net>
 *   Doron Rosenberg <doronr@us.ibm.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//  edited by ebricca 2004/04/16
//  came from the magpie / bkstr implementation from bengoodger to this
//  ideas also from the livehttpheaders -> observer

// A hxxp link looks like this:
// hxxp://normalUrl without referer
// alternatives h++p, h**p are implemented by page manipulation link generation


// hxxp protocol related
const kSCHEME = "hxxp";
const kPROTOCOL_NAME = "No Referer Protocol";
const kPROTOCOL_CONTRACTID = "@mozilla.org/network/protocol;1?name=" + kSCHEME;
const kPROTOCOL_CID = Components.ID("E93FA331-D6BD-11D8-9B81-00039388DA40");

// Mozilla defined
const kSIMPLEURI_CONTRACTID = "@mozilla.org/network/simple-uri;1";
const kIOSERVICE_CONTRACTID = "@mozilla.org/network/io-service;1";
const nsISupports = Components.interfaces.nsISupports;
const nsIIOService = Components.interfaces.nsIIOService;
const nsIProtocolHandler = Components.interfaces.nsIProtocolHandler;
const nsIURI = Components.interfaces.nsIURI;

var refnsiURI = null;

// logging
var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
function dumpme(aString)
{
 aConsoleService.logStringMessage(aString);
}

function Protocol()
{
}

Protocol.prototype =
{
  //


  QueryInterface: function(iid)
  {
    if (!iid.equals(nsIProtocolHandler) &&
        !iid.equals(nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;


    return this;
  },

  scheme: kSCHEME,
  defaultPort: -1,
  protocolFlags: nsIProtocolHandler.URI_NORELATIVE |
                 nsIProtocolHandler.URI_NOAUTH,
  
  allowPort: function(port, scheme)
  {
    return false;
  },

  newURI: function(spec, charset, baseURI)
  {
    var uri = Components.classes[kSIMPLEURI_CONTRACTID].createInstance(nsIURI);
    uri.spec = spec;   
    return uri;
  },

  newChannel: function(aURI)
  {
    var ios = Components.classes[kIOSERVICE_CONTRACTID].getService(nsIIOService);
    //hack against crash by providing a correct return channel
    var dumbnsiURI  = ios.newURI("http://www.innergeek.us/geek.html", null, null);
    var dumbChannel = ios.newChannelFromURI(dumbnsiURI).QueryInterface(Components.interfaces.nsIHttpChannel);


    // hack -> if we pass a https so the referrer gets cut per def
    var refURI = "https://www.tst.com";

    try {
      var ios = Components.classes[kIOSERVICE_CONTRACTID].getService(nsIIOService);
      refnsiURI  = ios.newURI(refURI, null, null);
      
      // also rewrites the url in the location bar back to http
      aURI.scheme = "http";
      aURI  = ios.newURI(aURI.spec, null, null);

    } catch(e) { // it isn't a valid url
      return dumbChannel;
    }

    // if it doesnt come from a link the ref is written here
    var myChannel = ios.newChannelFromURI(aURI).QueryInterface(Components.interfaces.nsIHttpChannel);
    myChannel.referrer=refnsiURI;


    // else if it comes from a link we call "http-on-modify-request"
    this.observe(null,"app-startup",null)

    return myChannel;

  },

  addObserver : function (observer)
  {
    this.observers[observer] = observer;
  },

  removeObserver : function (observer)
  {
    delete this.observers[observer];
  },

  observe: function(aSubject, aTopic, aData) {
    if (aTopic == 'http-on-modify-request') {
      aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
      this.onModifyRequest(aSubject);

    } else if (aTopic == 'app-startup') {
      if ('nsINetModuleMgr' in Components.interfaces) {
        dumpme("use a new moz version after september 15, 2003 \n");
        return;
      } else {
  var obsService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
        obsService.addObserver(this, "http-on-modify-request", false);       
        // Initialisation of all needed vars .. dont know how it works in detail yet
        this.observers = new Array();
        this.delays    = new Array();
      }
    } else {
        dumpme("nsHeaderInfo: unknown TOPIC: " + aTopic + "\n");
    }
  },

  onModifyRequest : function (oHttp)
  {
      oHttp.QueryInterface(Components.interfaces.nsIHttpChannel);
      oHttp.referrer=refnsiURI;
      // remove the obs afterwards ..
  var obsService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      obsService.removeObserver(this, "http-on-modify-request", false);
  },

}

var ProtocolFactory = new Object();

ProtocolFactory.createInstance = function (outer, iid)
{
  if (outer != null)
    throw Components.results.NS_ERROR_NO_AGGREGATION;

  if (!iid.equals(nsIProtocolHandler) &&
      !iid.equals(nsISupports))
    throw Components.results.NS_ERROR_NO_INTERFACE;

  return new Protocol();
}


/**
 * JS XPCOM component registration goop:
 *
 * We set ourselves up to observe the xpcom-startup category.  This provides
 * us with a starting point.
 */

var nsHxxpProtocolModule = new Object();

nsHxxpProtocolModule.registerSelf = function (compMgr, fileSpec, location, type)
{
  compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
  compMgr.registerFactoryLocation(kPROTOCOL_CID,
                                  kPROTOCOL_NAME,
                                  kPROTOCOL_CONTRACTID,
                                  fileSpec, 
                                  location, 
                                  type);
}

nsHxxpProtocolModule.getClassObject = function (compMgr, cid, iid)
{
  if (!cid.equals(kPROTOCOL_CID))
    throw Components.results.NS_ERROR_NO_INTERFACE;

  if (!iid.equals(Components.interfaces.nsIFactory))
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
  return ProtocolFactory;
}

nsHxxpProtocolModule.canUnload = function (compMgr)
{
  return true;
}

function NSGetModule(compMgr, fileSpec)
{
  return nsHxxpProtocolModule;
}
